#
# Copyright 2011-2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
#

import os
import textwrap
import ConfigParser

parameters = [
    # Section: [vars]
    ('vars', [

        ('core_dump_enable', 'true',
            'Enable core dump.'),

        ('host_mem_reserve', '256',
            'Reserves memory for the host to prevent VMs from using all the '
            'physical pages. The values are in Mbytes.'),

        ('guest_ram_overhead', '65', None),

        ('extra_mem_reserve', '65',
            'Memory reserved for non-vds-administered programs.'),

        ('fake_nics', 'dummy_*',
            'Comma-separated list of fnmatch-patterns for dummy hosts nics to '
            'be shown to vdsm.'),

        ('nic_model', 'rtl8139,pv',
            'NIC model is rtl8139, ne2k_pci pv or any other valid device '
            'recognized by kvm/qemu if a coma separated list given then a '
            'NIC per device will be created.'),

        ('migration_timeout', '300',
            'Maximum time the destination waits since migration is stalled. '
            'Please note, that this is not overall migration timeout. '
            'Source waits twice as long (to avoid races).'),

        ('migration_listener_timeout', '30',
            'Time to wait (in seconds) for migration destination to start '
            'listening before migration begins.'),

        ('migration_max_bandwidth', '32',
            'Maximum bandwidth for migration, in MiBps, 0 means libvirt\'s '
            'default, since 0.10.x default in libvirt is unlimited'),

        ('migration_monitor_interval', '10',
            'How often (in seconds) should the monitor thread pulse, 0 means '
            'the thread is disabled.'),

        ('hidden_nics', 'w*,usb*',
            'Comma-separated list of fnmatch-patterns for host nics to be '
            'hidden from vdsm.'),

        ('hidden_bonds', '',
            'Comma-separated list of fnmatch-patterns for host bonds to be '
            'hidden from vdsm.'),

        ('hidden_vlans', '',
            'Comma-separated list of fnmatch-patterns for host vlans to be '
            'hidden from vdsm. vlan names must be in the format "dev.VLANID" '
            '(e.g. eth0.100, em1.20, eth2.200). '
            'vlans with alternative names must be hidden from vdsm '
            '(e.g. eth0.10-fcoe, em1.myvlan100, vlan200)'),

        ('default_bridge', 'engine', None),

        ('migration_downtime', '500',
            'Maxmium allowed downtime for live migration in milliseconds '
            '(anything below 100ms is ignored) if you do not care about '
            'liveness of migration, set to a very high value, such as '
            '600000.'),

        ('migration_downtime_steps', '10',
            'Incremental steps used to reach migration_downtime.'),

        ('max_outgoing_migrations', '3',
            'Maximum concurrent outgoing migrations'),

        ('sys_shutdown_timeout', '10',
            'Destroy and shutdown timeouts (in sec) before completing the '
            'action.'),

        ('user_shutdown_timeout', '30',
            'Grace period (seconds) to let guest user close his '
            'applications before shutdown.'),

        ('guest_agent_timeout', '30',
            'Time (in sec) to wait for guest agent.'),

        ('vm_command_timeout', '60',
            'Time to wait (in seconds) for vm to respond to a monitor '
            'command, 30 secs is a nice default. Set to 300 if the vm is '
            'expected to freeze during cluster failover.'),

        ('vm_watermark_interval', '2',
            'How often should we sample each vm for statistics (seconds).'),

        ('vm_sample_cpu_interval', '15', None),

        ('vm_sample_cpu_window', '2', None),

        ('vm_sample_disk_interval', '60', None),

        ('vm_sample_disk_window', '2', None),

        ('vm_sample_disk_latency_interval', '60', None),

        ('vm_sample_disk_latency_window', '2', None),

        ('vm_sample_net_interval', '5', None),

        ('vm_sample_net_window', '2', None),

        ('trust_store_path', '/etc/pki/vdsm',
            'Where the certificates and keys are situated.'),

        ('ssl', 'true',
            'Whether to use ssl encryption and authentication.'),

        ('vds_responsiveness_timeout', '60', None),

        ('vdsm_nice', '-5', None),

        ('qemu_drive_cache', 'none', None),

        ('fake_kvm_support', 'false', None),

        ('xmlrpc_enable', 'true', 'Enable the xmlrpc server'),

        ('jsonrpc_enable', 'true', 'Enable the JSON RPC server'),

        ('report_host_threads_as_cores', 'false',
            'Count each cpu hyperthread as an individual core'),

        ('libvirt_env_variable_log_filters', '',
            'Specify the log filters to track libvirt calls'),

        ('libvirt_env_variable_log_outputs', '',
            'Specify the output to track libvirt calls'),
    ]),

    # Section: [ksm]
    ('ksm', [

        ('ksm_monitor_thread', 'true', None),

    ]),

    # Section: [mom]
    ('mom', [

        ('conf', '/etc/vdsm/mom.conf', 'mom configuration file'),

        ('tuning_policy', '01-parameters',
            'name of the mom policy to be updated from '
            'updatePolicyParameters API call'),

    ]),

    # Section: [irs]
    ('irs', [

        ('irs_enable', 'true', None),

        ('repository', '/rhev/data-center',
            'Image repository.'),

        ('hsm_tasks', '%(repository)s/hsm-tasks', None),

        ('images', '/images', None),

        ('irsd', '%(images)s/irsd', None),

        ('images_check_times', '0',
            'Image repository check period (seconds).'),

        ('volume_utilization_percent', '50', None),

        ('volume_utilization_chunk_mb', '1024', None),

        ('vol_size_sample_interval', '60',
            'How often should the volume size be checked (seconds).'),

        ('sd_validate_timeout', '80',
            'Storage domain validate timeout, the maximum number of seconds '
            'to wait until all the domains will be validated.'),

        ('scsi_rescan_minimal_timeout', '2',
            'The minimum number of seconds to wait for scsi scan to return.'),

        ('scsi_rescan_maximal_timeout', '30',
            'The maximal number of seconds to wait for scsi scan to return.'),

        ('sd_health_check_delay', '10',
            'Storage domain health check delay, the amount of seconds to '
            'wait between two successive run of the domain health check.'),

        ('nfs_mount_options', 'soft,nosharecache',
            'NFS mount options, comma-separated list (NB: no white space '
            'allowed!)'),

        ('pools_data_dir', '/var/run/vdsm/pools', None),

        ('vol_extend_policy', 'ON', None),

        ('lock_util_path', '/usr/libexec/vdsm', None),

        ('lock_cmd', 'spmprotect.sh', None),

        ('free_lock_cmd', 'spmstop.sh', None),

        ('thread_pool_size', '10', None),

        ('max_tasks', '500', None),

        ('lvm_dev_whitelist', '', None),

        ('md_backup_versions', '30', None),

        ('md_backup_dir', '/var/log/vdsm/backup', None),

        ('maximum_allowed_pvs', '8',
            'The number of PVs per VG has a hard-coded limit of 10.'),

        ('repo_stats_cache_refresh_timeout', '300', None),

        ('task_resource_default_timeout', '120000', None),

        ('prepare_image_timeout', '600000', None),

        ('gc_blocker_force_collect_interval', '60', None),

        ('maximum_domains_in_pool', '100',
            'Process pool configuration.'),

        ('process_pool_size', '100', None),

        ('process_pool_timeout', '60', None),

        ('process_pool_grace_period', '2', None),

        ('process_pool_max_slots_per_domain', '10', None),

        ('iscsi_default_ifaces', 'default',
            'Comma seperated ifaces to connect with. '
            'i.e. iser,default'),

        ('use_volume_leases', 'false',
            'Whether to use the volume leases or not.'),
    ]),

    # Section: [addresses]
    ('addresses', [

        ('management_port', '54321',
            'Port on which the vdsmd XMPRPC server listens to network '
            'clients.'),

        ('json_port', '4044',
            'Port on which the vdsmd Json RPC server listens to network '
            'clients.'),

        ('management_ip', '0.0.0.0', 'Set to "::" to listen on IPv6.'),

        ('guests_gateway_ip', '', None),

    ]),
]


def __prepare_config():
    global config

    for section, keylist in parameters:
        config.add_section(section)
        for key, value, comment in keylist:
            config.set(section, key, value)


def print_config():
    twp = textwrap.TextWrapper(initial_indent='# ', subsequent_indent='# ')

    print twp.fill("VDSM Sample Configuration")
    print

    for section, keylist in parameters:
        print "[%s]\n" % section

        for key, value, comment in keylist:
            if comment:
                print twp.fill(comment)
            print twp.fill("%s = %s" % (key, value))
            print

config = ConfigParser.ConfigParser()
__prepare_config()
config.read([os.path.join('/etc/vdsm', 'vdsm.conf')])

if __name__ == '__main__':
    print_config()
